@@ -27,30 +27,35 @@ import re
 import textwrap
 from collections import OrderedDict, defaultdict
 from shutil import get_terminal_size
-from typing import Iterable, List, Tuple
+from typing import Callable, List, Optional
 
 import pytz
 from click import confirm, echo, prompt, style
 
-from khal import (__productname__, __version__, calendar_display,
-                  parse_datetime, utils)
+from khal import __productname__, __version__, calendar_display, parse_datetime
+from khal.custom_types import (
+    EventCreationTypes,
+    LocaleConfiguration,
+    MonthDisplayType,
+    WeekNumbersType,
+)
 from khal.exceptions import DateTimeParseError, FatalError
 from khal.khalendar import CalendarCollection
 from khal.khalendar.event import Event
 from khal.khalendar.exceptions import DuplicateUid, ReadOnlyCalendarError
 
 from .exceptions import ConfigurationError
-from .icalendar import cal_from_ics
-from .icalendar import new_event as new_vevent
+from .icalendar import cal_from_ics, split_ics
 from .icalendar import sort_key as sort_vevent_key
-from .icalendar import split_ics
 from .khalendar.vdir import Item
+from .parse_datetime import timedelta2str
 from .terminal import merge_columns
+from .utils import human_formatter, json_formatter
 
 logger = logging.getLogger('khal')
 
 
-def format_day(day, format_string: str, locale, attributes=None):
+def format_day(day: dt.date, format_string: str, locale, attributes=None):
     if attributes is None:
         attributes = {}
 
@@ -70,23 +75,29 @@ def format_day(day, format_string: str, locale, attributes=None):
         raise KeyError("cannot format day with: %s" % format_string)
 
 
-def calendar(collection, agenda_format=None, notstarted=False, once=False, daterange=None,
-             day_format=None,
-             locale=None,
-             conf=None,
-             firstweekday=0,
-             weeknumber=False,
-             monthdisplay='firstday',
-             hmethod='fg',
-             default_color='',
-             multiple='',
-             multiple_on_overflow=False,
-             color='',
-             highlight_event_days=0,
-             full=False,
-             bold_for_light_color=True,
-             env=None,
-             ):
+def calendar(
+    collection: CalendarCollection,
+    agenda_format=None,
+    notstarted: bool=False,
+    once=False,
+    daterange=None,
+    day_format=None,
+    locale=None,
+    conf=None,
+    firstweekday: int=0,
+    weeknumber: WeekNumbersType=False,
+    monthdisplay: MonthDisplayType='firstday',
+    hmethod: str='fg',
+    default_color: str='',
+    multiple='',
+    multiple_on_overflow: bool=False,
+    color='',
+    highlight_event_days=0,
+    full=False,
+    bold_for_light_color: bool=True,
+    env=None,
+    ):
+
     term_width, _ = get_terminal_size()
     lwidth = 27 if conf['locale']['weeknumbers'] == 'right' else 25
     rwidth = term_width - lwidth - 4
@@ -113,10 +124,11 @@ def calendar(collection, agenda_format=None, notstarted=False, once=False, dater
     )
     if not event_column:
         event_column = [style('No events', bold=True)]
+    month_count = (end.year - start.year) * 12 + end.month - start.month + 1
     calendar_column = calendar_display.vertical_month(
         month=start.month,
         year=start.year,
-        count=max(3, (end.year - start.year) * 12 + end.month - start.month + 1),
+        count=max(conf['view']['min_calendar_display'], month_count),
         firstweekday=firstweekday, weeknumber=weeknumber,
         monthdisplay=monthdisplay,
         collection=collection,
@@ -132,11 +144,11 @@ def calendar(collection, agenda_format=None, notstarted=False, once=False, dater
 
 
 def start_end_from_daterange(
-    daterange: Iterable[str],
-    locale: dict,
-    default_timedelta_date=dt.timedelta(days=1),
-    default_timedelta_datetime=dt.timedelta(hours=1),
-) -> Tuple[dt.datetime, dt.datetime]:
+    daterange: List[str],
+    locale: LocaleConfiguration,
+    default_timedelta_date: dt.timedelta=dt.timedelta(days=1),
+    default_timedelta_datetime: dt.timedelta=dt.timedelta(hours=1),
+):
     """
     convert a string description of a daterange into start and end datetime
 
@@ -161,24 +173,27 @@ def get_events_between(
     locale: dict,
     start: dt.datetime,
     end: dt.datetime,
-    agenda_format: str,
+    formatter: Callable,
     notstarted: bool,
     env: dict,
-    width,
-    seen,
     original_start: dt.datetime,
+    seen=None,
+    colors: bool = True,
 ) -> List[str]:
     """returns a list of events scheduled between start and end. Start and end
     are strings or datetimes (of some kind).
 
     :param collection:
+    :param locale:
     :param start: the start datetime
     :param end: the end datetime
-    :param agenda_format: a format string that can be used in python string formatting
-    :param env: a collection of "static" values like calendar names and color
+    :param formatter: the formatter (see :class:`.utils.human_formatter`)
     :param nostarted: True if each event should start after start (instead of
-    be active between start and end)
+      be active between start and end)
+    :param env: a collection of "static" values like calendar names and color
     :param original_start: start datetime to compare against of notstarted is set
+    :param seen:
+    :param colors:
     :returns: a list to be printed as the agenda for the given days
     """
     assert not (notstarted and not original_start)
@@ -209,31 +224,29 @@ def get_events_between(
             continue
 
         try:
-            event_string = event.format(agenda_format, relative_to=(start, end), env=env)
+            event_attributes = event.attributes(relative_to=(start, end), env=env, colors=colors)
         except KeyError as error:
             raise FatalError(error)
 
-        if width:
-            event_list += utils.color_wrap(event_string, width)
-        else:
-            event_list.append(event_string)
+        event_list.append(event_attributes)
         if seen is not None:
             seen.add(event.uid)
 
-    return event_list
+    return formatter(event_list)
 
 
 def khal_list(
     collection,
-    daterange: Iterable[str] = None,
-    conf: dict = None,
+    daterange: Optional[List[str]] = None,
+    conf: Optional[dict] = None,
     agenda_format=None,
-    day_format: str=None,
+    day_format: Optional[str]=None,
     once=False,
     notstarted: bool = False,
-    width: bool = False,
+    width: Optional[int] = None,
     env=None,
     datepoint=None,
+    json: Optional[List] = None,
 ):
     """returns a list of all events in `daterange`"""
     assert daterange is not None or datepoint is not None
@@ -243,6 +256,13 @@ def khal_list(
     if agenda_format is None:
         agenda_format = conf['view']['agenda_event_format']
 
+    if json:
+        formatter = json_formatter(json)
+        colors = False
+    else:
+        formatter = human_formatter(agenda_format, width)
+        colors = True
+
     if daterange is not None:
         if day_format is None:
             day_format = conf['view']['agenda_day_format']
@@ -257,11 +277,13 @@ def khal_list(
         if not datepoint:
             datepoint = ['now']
         try:
+            # hand over a copy of the `datepoint` so error reporting works
+            # (we pop from that list in guessdatetimefstr())
             start, allday = parse_datetime.guessdatetimefstr(
-                datepoint, conf['locale'], dt.date.today(),
+                list(datepoint), conf['locale'], dt.date.today(),
             )
-        except ValueError:
-            raise FatalError('Invalid value of `{' '.join(datepoint)}` for a datetime')
+        except (ValueError, IndexError):
+            raise FatalError(f"Invalid value of {' '.join(datepoint)} for a datetime")
         if allday:
             logger.debug(f'Got date {start}')
             raise FatalError('Please supply a datetime, not a date.')
@@ -285,13 +307,13 @@ def khal_list(
         else:
             day_end = dt.datetime.combine(start.date(), dt.time.max)
         current_events = get_events_between(
-            collection, locale=conf['locale'], agenda_format=agenda_format, start=start,
+            collection, locale=conf['locale'], formatter=formatter, start=start,
             end=day_end, notstarted=notstarted, original_start=original_start,
             env=env,
             seen=once,
-            width=width,
+            colors=colors,
         )
-        if day_format and (conf['default']['show_all_days'] or current_events):
+        if day_format and (conf['default']['show_all_days'] or current_events) and not json:
             if len(event_column) != 0 and conf['view']['blank_line_before_day']:
                 event_column.append('')
             event_column.append(format_day(start.date(), day_format, conf['locale']))
@@ -303,13 +325,14 @@ def khal_list(
 
 def new_interactive(collection, calendar_name, conf, info, location=None,
                     categories=None, repeat=None, until=None, alarms=None,
-                    format=None, env=None, url=None):
+                    format=None, json=None, env=None, url=None):
+    info: EventCreationTypes
     try:
         info = parse_datetime.eventinfofstr(
             info, conf['locale'],
-            conf['default']['default_event_duration'],
-            conf['default']['default_dayevent_duration'],
-            adjust_reasonably=True, localize=False,
+            default_event_duration=conf['default']['default_event_duration'],
+            default_dayevent_duration=conf['default']['default_dayevent_duration'],
+            adjust_reasonably=True,
         )
     except DateTimeParseError:
         info = {}
@@ -353,12 +376,24 @@ def new_interactive(collection, calendar_name, conf, info, location=None,
     if info['description'] == 'None':
         info['description'] = ''
 
-    event = new_from_args(
-        collection, calendar_name, conf, format=format, env=env,
-        location=location, categories=categories,
-        repeat=repeat, until=until, alarms=alarms, url=url,
-        **info)
+    info.update({
+        'location': location,
+        'categories': categories,
+        'repeat': repeat,
+        'until': until,
+        'alarms': alarms,
+        'url': url,
+    })
 
+    event = new_from_dict(
+        info,
+        collection,
+        conf,
+        format=format,
+        env=env,
+        calendar_name=calendar_name,
+        json=json,
+    )
     echo("event saved")
 
     term_width, _ = get_terminal_size()
@@ -367,55 +402,77 @@ def new_interactive(collection, calendar_name, conf, info, location=None,
 
 def new_from_string(collection, calendar_name, conf, info, location=None,
                     categories=None, repeat=None, until=None, alarms=None,
-                    url=None, format=None, env=None):
+                    url=None, format=None, json=None, env=None):
     """construct a new event from a string and add it"""
     info = parse_datetime.eventinfofstr(
         info, conf['locale'],
         conf['default']['default_event_duration'],
         conf['default']['default_dayevent_duration'],
-        adjust_reasonably=True, localize=False
+        adjust_reasonably=True,
     )
-    new_from_args(
-        collection, calendar_name, conf, format=format, env=env,
-        location=location, categories=categories, repeat=repeat,
-        until=until, alarms=alarms, url=url, **info
+    if alarms is None:
+        if info['allday']:
+            alarms = timedelta2str(conf['default']['default_dayevent_alarm'])
+        else:
+            alarms = timedelta2str(conf['default']['default_event_alarm'])
+    info.update({
+        'location': location,
+        'categories': categories,
+        'repeat': repeat,
+        'until': until,
+        'alarms': alarms,
+        'url': url,
+    })
+    new_from_dict(
+        info,
+        collection,
+        conf=conf,
+        format=format,
+        env=env,
+        calendar_name=calendar_name,
+        json=json,
     )
 
 
-def new_from_args(collection, calendar_name, conf, dtstart=None, dtend=None,
-                  summary=None, description=None, allday=None, location=None,
-                  categories=None, repeat=None, until=None, alarms=None,
-                  timezone=None, url=None, format=None, env=None):
-    """Create a new event from arguments and add to vdirs"""
-    if isinstance(categories, str):
-        categories = [category.strip() for category in categories.split(',')]
+def new_from_dict(
+    event_args: EventCreationTypes,
+    collection: CalendarCollection,
+    conf,
+    calendar_name: Optional[str]=None,
+    format=None,
+    env=None,
+    json=None,
+) -> Event:
+    """Create a new event from arguments and save in vdirs
+
+    This is a wrapper around CalendarCollection.create_event_from_dict()
+    """
+    if isinstance(event_args['categories'], str):
+        event_args['categories'] = [event_args['categories'].strip()
+                                    for category in event_args['categories'].split(',')]
     try:
-        event = new_vevent(
-            locale=conf['locale'], location=location, categories=categories,
-            repeat=repeat, until=until, alarms=alarms, dtstart=dtstart,
-            dtend=dtend, summary=summary, description=description, timezone=timezone, url=url,
-        )
+        event = collection.create_event_from_dict(event_args, calendar_name=calendar_name)
     except ValueError as error:
         raise FatalError(error)
-    event = Event.fromVEvents(
-        [event], calendar=calendar_name, locale=conf['locale'])
 
     try:
-        collection.new(event)
+        collection.insert(event)
     except ReadOnlyCalendarError:
         raise FatalError(
-            f'ERROR: Cannot modify calendar "{calendar_name}" as it is read-only'
+            f'ERROR: Cannot modify calendar `{calendar_name}` as it is read-only'
         )
 
     if conf['default']['print_new'] == 'event':
-        if format is None:
-            format = conf['view']['event_format']
-        echo(event.format(format, dt.datetime.now(), env=env))
+        if json is None or len(json) == 0:
+            if format is None:
+                format = conf['view']['event_format']
+            formatter = human_formatter(format)
+        else:
+            formatter = json_formatter(json)
+        echo(formatter(event.attributes(dt.datetime.now(), env=env)))
     elif conf['default']['print_new'] == 'path':
-        path = os.path.join(
-            collection._calendars[event.calendar]['path'],
-            event.href
-        )
+        assert event.href
+        path = os.path.join(collection._calendars[event.calendar]['path'], event.href)
         echo(path)
     return event
 
@@ -477,7 +534,7 @@ def edit_event(event, collection, locale, allow_quit=False, width=80):
                 collection.delete(event.href, event.etag, event.calendar)
                 return True
         elif choice == "datetime range":
-            current = event.format("{start} {end}", relative_to=now)
+            current = human_formatter("{start} {end}")(event.attributes(relative_to=now))
             value = prompt("datetime range", default=current)
             try:
                 start, end, allday = parse_datetime.guessrangefstr(ansi.sub('', value), locale)
@@ -561,7 +618,8 @@ def edit(collection, search_string, locale, format=None, allow_past=False, conf=
                 continue
             elif not event.allday and event.end_local < now:
                 continue
-        event_text = textwrap.wrap(event.format(format, relative_to=now), term_width)
+        event_text = textwrap.wrap(human_formatter(format)(
+            event.attributes(relative_to=now)), term_width)
         echo(''.join(event_text))
         if not edit_event(event, collection, locale, allow_quit=True, width=term_width):
             return
@@ -592,7 +650,10 @@ def import_ics(collection, conf, ics, batch=False, random_uid=False, format=None
     """
     if format is None:
         format = conf['view']['event_format']
-    vevents = split_ics(ics, random_uid, conf['locale']['default_timezone'])
+    try:
+        vevents = split_ics(ics, random_uid, conf['locale']['default_timezone'])
+    except Exception as error:
+        raise FatalError(error)
     for vevent in vevents:
         import_event(vevent, collection, conf['locale'], batch, format, env)
 
@@ -610,7 +671,7 @@ def import_event(vevent, collection, locale, batch, format=None, env=None):
             if item.name == 'VEVENT':
                 event = Event.fromVEvents(
                     [item], calendar=collection.default_calendar_name, locale=locale)
-                echo(event.format(format, dt.datetime.now(), env=env))
+                echo(human_formatter(format)(event.attributes(dt.datetime.now(), env=env)))
 
     # get the calendar to insert into
     if not collection.writable_names:
@@ -642,7 +703,7 @@ def import_event(vevent, collection, locale, batch, format=None, env=None):
 
     if batch or confirm(f"Do you want to import this event into `{calendar_name}`?"):
         try:
-            collection.new(Item(vevent), collection=calendar_name)
+            collection.insert(Item(vevent), collection=calendar_name)
         except DuplicateUid:
             if batch or confirm(
                     "An event with the same UID already exists. Do you want to update it?"):
@@ -667,4 +728,4 @@ def print_ics(conf, name, ics, format):
     echo(f'{len(vevents)} events found in {name}')
     for sub_event in vevents:
         event = Event.fromVEvents(sub_event, locale=conf['locale'])
-        echo(event.format(format, dt.datetime.now()))
+        echo(human_formatter(format)(event.attributes(dt.datetime.now())))
