@@ -25,27 +25,25 @@ import datetime as dt
 import logging
 from collections import defaultdict
 from hashlib import sha256
+from typing import List, Optional, Tuple, Union
 
 import dateutil.rrule
 import icalendar
 import pytz
 
 from .exceptions import UnsupportedRecurrence
-from .parse_datetime import guesstimedeltafstr, rrulefstr
-from .utils import generate_random_uid, localize_strip_tz, to_unix_time
+from .parse_datetime import rrulefstr
+from .utils import generate_random_uid, localize_strip_tz, str2alarm, to_unix_time
 
 logger = logging.getLogger('khal')
 
 
-def split_ics(ics, random_uid=False, default_timezone=None):
+def split_ics(ics: str, random_uid: bool=False, default_timezone=None) -> List:
     """split an ics string into several according to VEVENT's UIDs
 
     and sort the right VTIMEZONEs accordingly
     ignores all other ics components
-    :type ics: str
     :param random_uid: assign random uids to all events
-    :type random_uid: bool
-    :rtype list:
     """
     cal = cal_from_ics(ics)
     tzs = {}
@@ -73,42 +71,51 @@ def split_ics(ics, random_uid=False, default_timezone=None):
             events_grouped[item['UID']].append(item)
         else:
             continue
-    return [ics_from_list(events, tzs, random_uid, default_timezone) for uid, events in
-            sorted(events_grouped.items())]
+    out = []
+    saved_exception = None
+    for uid, events in sorted(events_grouped.items()):
+        try:
+            ics = ics_from_list(events, tzs, random_uid, default_timezone)
+        except Exception as exception:
+            logger.warn(f'Error when trying to import the event {uid}')
+            saved_exception = exception
+        else:
+            out.append(ics)
+    if saved_exception:
+        raise saved_exception
+    return out
 
 
-def new_event(locale, dtstart=None, dtend=None, summary=None, timezone=None,
-              allday=False, description=None, location=None, categories=None,
-              repeat=None, until=None, alarms=None, url=None):
+def new_vevent(locale,
+               dtstart: dt.date,
+               dtend: dt.date,
+               summary: str,
+               timezone: Optional[pytz.BaseTzInfo]=None,
+               allday: bool=False,
+               description: Optional[str]=None,
+               location: Optional[str]=None,
+               categories: Optional[Union[List[str], str]]=None,
+               repeat: Optional[str]=None,
+               until=None,
+               alarms: Optional[str]=None,
+               url: Optional[str]=None,
+               ) -> icalendar.Event:
     """create a new event
 
     :param dtstart: starttime of that event
-    :type dtstart: datetime
     :param dtend: end time of that event, if this is a *date*, this value is
         interpreted as being the last date the event is scheduled on, i.e.
         the VEVENT DTEND will be *one day later*
-    :type dtend: datetime
     :param summary: description of the event, used in the SUMMARY property
-    :type summary: unicode
     :param timezone: timezone of the event (start and end)
-    :type timezone: pytz.timezone
     :param allday: if set to True, we will not transform dtstart and dtend to
         datetime
-    :type allday: bool
     :param url: url of the event
-    :type url: string
     :returns: event
-    :rtype: icalendar.Event
     """
-
-    if dtstart is None:
-        raise ValueError("no start given")
-    if dtend is None:
-        raise ValueError("no end given")
-    if summary is None:
-        raise ValueError("no summary given")
-
     if not allday and timezone is not None:
+        assert isinstance(dtstart, dt.datetime)
+        assert isinstance(dtend, dt.datetime)
         dtstart = timezone.localize(dtstart)
         dtend = timezone.localize(dtend)
 
@@ -129,27 +136,24 @@ def new_event(locale, dtstart=None, dtend=None, summary=None, timezone=None,
     if url:
         event.add('url', icalendar.vUri(url))
     if repeat and repeat != "none":
-        rrule = rrulefstr(repeat, until, locale, dtstart.tzinfo)
+        rrule = rrulefstr(repeat, until, locale, getattr(dtstart, 'tzinfo', None))
         event.add('rrule', rrule)
     if alarms:
-        for alarm in alarms.split(","):
-            alarm = alarm.strip()
-            alarm_trig = -1 * guesstimedeltafstr(alarm)
-            new_alarm = icalendar.Alarm()
-            new_alarm.add('ACTION', 'DISPLAY')
-            new_alarm.add('TRIGGER', alarm_trig)
-            new_alarm.add('DESCRIPTION', description)
-            event.add_component(new_alarm)
+        for alarm in str2alarm(alarms, description or ''):
+            event.add_component(alarm)
     return event
 
 
-def ics_from_list(events, tzs, random_uid=False, default_timezone=None):
-    """convert an iterable of icalendar.Events to an icalendar.Calendar
+def ics_from_list(
+    events: List[icalendar.Event],
+    tzs,
+    random_uid: bool=False,
+    default_timezone=None
+) -> str:
+    """convert an iterable of icalendar.Events to an icalendar str
 
     :params events: list of events all with the same uid
-    :type events: list(icalendar.cal.Event)
     :param random_uid: assign random uids to all events
-    :type random_uid: bool
     :param tzs: collection of timezones
     :type tzs: dict(icalendar.cal.Vtimzone
     """
@@ -206,7 +210,10 @@ def ics_from_list(events, tzs, random_uid=False, default_timezone=None):
     return calendar.to_ical().decode('utf-8')
 
 
-def expand(vevent, href=''):
+def expand(
+    vevent: icalendar.Event,
+    href: str='',
+) -> Optional[List[Tuple[dt.datetime, dt.datetime]]]:
     """
     Constructs a list of start and end dates for all recurring instances of the
     event defined in vevent.
@@ -218,12 +225,9 @@ def expand(vevent, href=''):
     the function still returns a tuple of start and end (date)times.
 
     :param vevent: vevent to be expanded
-    :type vevent: icalendar.cal.Event
     :param href: the href of the vevent, used for more informative logging and
                  nothing else
-    :type href: str
     :returns: list of start and end (date)times of the expanded event
-    :rtype: list(tuple(datetime, datetime))
     """
     # we do this now and than never care about the "real" end time again
     if 'DURATION' in vevent:
@@ -238,7 +242,7 @@ def expand(vevent, href=''):
     events_tz = getattr(vevent['DTSTART'].dt, 'tzinfo', None)
     allday = not isinstance(vevent['DTSTART'].dt, dt.datetime)
 
-    def sanitize_datetime(date):
+    def sanitize_datetime(date: dt.date) -> dt.date:
         if allday and isinstance(date, dt.datetime):
             date = date.date()
         if events_tz is not None:
@@ -263,20 +267,24 @@ def expand(vevent, href=''):
             ignoretz=True,
         )
 
-        if rrule._until is None:
+        # telling mypy, that _until exists
+        # we are very sure (TM) that rrulestr always returns a rrule, not a
+        # rruleset (which wouldn't have a _until attribute)
+        if rrule._until is None:  # type: ignore
             # rrule really doesn't like to calculate all recurrences until
             # eternity, so we only do it until 2037, because a) I'm not sure
             # if python can deal with larger datetime values yet and b) pytz
             # doesn't know any larger transition times
-            rrule._until = dt.datetime(2037, 12, 31)
+            rrule._until = dt.datetime(2037, 12, 31)  # type: ignore
         else:
             if events_tz and 'Z' in rrule_param.to_ical().decode():
-                rrule._until = pytz.UTC.localize(
-                    rrule._until).astimezone(events_tz).replace(tzinfo=None)
+                assert isinstance(rrule._until, dt.datetime)  # type: ignore
+                rrule._until = pytz.UTC.localize(  # type: ignore
+                    rrule._until).astimezone(events_tz).replace(tzinfo=None)  # type: ignore
 
             # rrule._until and dtstart could be dt.date or dt.datetime. They
             # need to be the same for comparison
-            testuntil = rrule._until
+            testuntil = rrule._until  # type: ignore
             if (type(dtstart) == dt.date and type(testuntil) == dt.datetime):
                 testuntil = testuntil.date()
             teststart = dtstart
@@ -287,15 +295,15 @@ def expand(vevent, href=''):
                 logger.warning(
                     f'{href}: Unsupported recurrence. UNTIL is before DTSTART.\n'
                     'This event will not be available in khal.')
-                return False
+                return None
 
         if rrule.count() == 0:
             logger.warning(
                 f'{href}: Recurrence defined but will never occur.\n'
                 'This event will not be available in khal.')
-            return False
+            return None
 
-        rrule = map(sanitize_datetime, rrule)
+        rrule = map(sanitize_datetime, rrule)  # type: ignore
 
         logger.debug(f'calculating recurrence dates for {href}, this might take some time.')
 
@@ -340,7 +348,7 @@ def expand(vevent, href=''):
 
 
 def assert_only_one_uid(cal: icalendar.Calendar):
-    """assert the all VEVENTs in cal have the same UID"""
+    """assert that all VEVENTs in cal have the same UID"""
     uids = set()
     for item in cal.walk():
         if item.name == 'VEVENT':
@@ -351,24 +359,24 @@ def assert_only_one_uid(cal: icalendar.Calendar):
         return True
 
 
-def sanitize(vevent, default_timezone, href='', calendar=''):
+def sanitize(
+    vevent: icalendar.Event,
+    default_timezone: pytz.BaseTzInfo,
+    href: str='',
+    calendar: str='',
+) -> icalendar.Event:
     """
     clean up vevents we do not understand
 
     :param vevent: the vevent that needs to be cleaned
-    :type vevent: icalendar.cal.Event
     :param default_timezone: timezone to apply to start and/or end dates which
          were supposed to be localized but which timezone was not understood
          by icalendar
-    :type timezone: pytz.timezone
     :param href: used for logging to inform user which .ics files are
         problematic
-    :type href: str
     :param calendar: used for logging to inform user which .ics files are
         problematic
-    :type calendar: str
     :returns: clean vevent
-    :rtype: icalendar.cal.Event
     """
     # convert localized datetimes with timezone information we don't
     # understand to the default timezone
@@ -418,6 +426,9 @@ def sanitize_timerange(dtstart, dtend, duration=None):
                 "Assuming it's the same timezone as the end time"
             )
             dtstart = dtend.tzinfo.localize(dtstart)
+    if dtend is not None and type(dtstart) != type(dtend):
+        raise ValueError(
+            'The event\'s end time (DTEND) and start time (DTSTART) are not of the same type.')
 
     if dtend is None and duration is None:
         if isinstance(dtstart, dt.datetime):
@@ -481,12 +492,9 @@ def _get_all_properties(vevent, prop):
     return rdates
 
 
-def delete_instance(vevent, instance):
+def delete_instance(vevent: icalendar.Event, instance: dt.datetime) -> None:
     """remove a recurrence instance from a VEVENT's RRDATE list or add it
     to the EXDATE list
-
-    :type vevent: icalendar.cal.Event
-    :type instance: datetime.datetime
     """
     # TODO check where this instance is coming from and only call the
     # appropriate function
@@ -502,7 +510,7 @@ def delete_instance(vevent, instance):
             vevent.add('RDATE', rdates)
 
 
-def sort_key(vevent):
+def sort_key(vevent: icalendar.Event) -> Tuple[str, float]:
     """helper function to determine order of VEVENTS
     so that recurrence-id events come after the corresponding rrule event, etc
     :param vevent: icalendar.Event
@@ -520,7 +528,10 @@ def sort_key(vevent):
         return uid, 1
 
 
-def cal_from_ics(ics):
+def cal_from_ics(ics: str) -> icalendar.Calendar:
+    """
+    :param ics: an icalendar formatted string
+    """
     try:
         cal = icalendar.Calendar.from_ical(ics)
     except ValueError as error:
@@ -533,4 +544,6 @@ def cal_from_ics(ics):
             icalendar.vUTCOffset.ignore_exceptions = True
             cal = icalendar.Calendar.from_ical(ics)
             icalendar.vUTCOffset.ignore_exceptions = False
+        else:
+            raise
     return cal
