@@ -25,15 +25,20 @@ helper functions."""
 import datetime as dt
 import logging
 import os
+from typing import Callable, Dict, List, Optional, Tuple, Type, Union
 
 import icalendar
+import icalendar.cal
+import icalendar.prop
 import pytz
 from click import style
+from pytz.tzinfo import StaticTzInfo
 
+from ..custom_types import LocaleConfiguration
 from ..exceptions import FatalError
 from ..icalendar import cal_from_ics, delete_instance, invalid_timezone
 from ..parse_datetime import timedelta2str
-from ..terminal import get_color
+from ..plugins import FORMATTERS
 from ..utils import generate_random_uid, is_aware, to_naive_utc, to_unix_time
 
 logger = logging.getLogger('khal')
@@ -43,7 +48,7 @@ class Event:
     """base Event class for representing a *recurring instance* of an Event
 
     (in case of non-recurring events this distinction is irrelevant)
-    We keep a copy of the start and end time around, because for recurring
+    We keep a copy of this instances start and end time around, because for recurring
     events it might be costly to expand the recursion rules
 
     important distinction for AllDayEvents:
@@ -51,28 +56,40 @@ class Event:
         only one day will have the same start and end date (even though the
         icalendar standard would have the end date be one day later)
     """
-    allday = False
+    allday: bool = False
 
-    def __init__(self, vevents, ref=None, **kwargs):
+    def __init__(self,
+                 vevents: Dict[str, icalendar.Event],
+                 locale: LocaleConfiguration,
+                 ref: Optional[str] = None,
+                 readonly: bool = False,
+                 href: Optional[str] = None,
+                 etag: Optional[str] = None,
+                 calendar: Optional[str] = None,
+                 color: Optional[str] = None,
+                 start: Optional[dt.datetime] = None,
+                 end: Optional[dt.datetime] = None,
+                 addresses: Optional[List[str]] =None,
+                 ):
         """
         :param start: start datetime of this event instance
-        :type start: datetime.date
-        :param end: end datetime of this event instance in unix time
-        :type end: datetime.date
+        :param end: end datetime of this event instance
         """
         if self.__class__.__name__ == 'Event':
             raise ValueError('do not initialize this class directly')
+        if ref is None:
+            raise ValueError('ref should not be None')
         self._vevents = vevents
-        self._locale = kwargs.pop('locale', None)
-        self.readonly = kwargs.pop('readonly', None)
-        self.href = kwargs.pop('href', None)
-        self.etag = kwargs.pop('etag', None)
-        self.calendar = kwargs.pop('calendar', None)
-        self.color = kwargs.pop('color', None)
         self.ref = ref
-
-        start = kwargs.pop('start', None)
-        end = kwargs.pop('end', None)
+        self._locale = locale
+        self.readonly = readonly
+        self.href = href
+        self.etag = etag
+        self.calendar = calendar if calendar else ''
+        self.color = color
+        self._start: dt.datetime
+        self._end: dt.datetime
+        self.addresses = addresses if addresses else []
 
         if start is None:
             self._start = self._vevents[self.ref]['DTSTART'].dt
@@ -86,18 +103,12 @@ class Event:
                     self._end = self._start + self._vevents[self.ref]['DURATION'].dt
                 except KeyError:
                     self._end = self._start + dt.timedelta(days=1)
-
         else:
             self._end = end
-        if kwargs:
-            raise TypeError('%s are invalid keyword arguments to this function' % kwargs.keys())
 
     @classmethod
-    def _get_type_from_vDDD(cls, start):
-        """
-        :type start: icalendar.prop.vDDDTypes
-        :type start: icalendar.prop.vDDDTypes
-        """
+    def _get_type_from_vDDD(cls, start: icalendar.prop.vDDDTypes) -> type:
+        """infere the type of the class from the START type of the event"""
         if not isinstance(start.dt, dt.datetime):
             return AllDayEvent
         if 'TZID' in start.params or start.dt.tzinfo is not None:
@@ -105,7 +116,7 @@ class Event:
         return FloatingEvent
 
     @classmethod
-    def _get_type_from_date(cls, start):
+    def _get_type_from_date(cls, start: dt.datetime) -> Type['Event']:
         if hasattr(start, 'tzinfo') and start.tzinfo is not None:
             cls = LocalizedEvent
         elif isinstance(start, dt.datetime):
@@ -115,10 +126,11 @@ class Event:
         return cls
 
     @classmethod
-    def fromVEvents(cls, events_list, ref=None, **kwargs):
-        """
-        :type events: list
-        """
+    def fromVEvents(cls,
+                    events_list: List[icalendar.Event],
+                    ref: Optional[str]=None,
+                    start: Optional[dt.datetime]=None,
+                    **kwargs) -> 'Event':
         assert isinstance(events_list, list)
 
         vevents = {}
@@ -142,19 +154,19 @@ class Event:
         except KeyError:
             pass
 
-        if kwargs.get('start'):
-            instcls = cls._get_type_from_date(kwargs.get('start'))
+        if start:
+            instcls = cls._get_type_from_date(start)
         else:
             instcls = cls._get_type_from_vDDD(vevents[ref]['DTSTART'])
-        return instcls(vevents, ref=ref, **kwargs)
+        return instcls(vevents, ref=ref, start=start, **kwargs)
 
     @classmethod
-    def fromString(cls, event_str, ref=None, **kwargs):
-        calendar_collection = cal_from_ics(event_str)
+    def fromString(cls, ics: str, ref=None, **kwargs) -> 'Event':
+        calendar_collection = cal_from_ics(ics)
         events = [item for item in calendar_collection.walk() if item.name == 'VEVENT']
         return cls.fromVEvents(events, ref, **kwargs)
 
-    def __lt__(self, other):
+    def __lt__(self, other: 'Event') -> bool:
         start = self.start_local
         other_start = other.start_local
         if isinstance(start, dt.date) and not isinstance(start, dt.datetime):
@@ -191,7 +203,7 @@ class Event:
         except TypeError:
             raise ValueError(f'Cannot compare events {start} and {other_start}')
 
-    def update_start_end(self, start, end):
+    def update_start_end(self, start: dt.datetime, end: dt.datetime) -> None:
         """update start and end time of this event
 
         calling this on a recurring event will lead to the proto instance
@@ -199,7 +211,7 @@ class Event:
 
         beware, this methods performs some open heart surgery
         """
-        if type(start) != type(end):  # flake8: noqa
+        if type(start) != type(end):
             raise ValueError('DTSTART and DTEND should be of the same type (datetime or date)')
         self.__class__ = self._get_type_from_date(start)
 
@@ -217,7 +229,7 @@ class Event:
             self._vevents[self.ref].add('DURATION', end - start)
 
     @property
-    def recurring(self):
+    def recurring(self) -> bool:
         try:
             rval = 'RRULE' in self._vevents[self.ref] or \
                 'RECURRENCE-ID' in self._vevents[self.ref] or \
@@ -232,26 +244,26 @@ class Event:
             return rval
 
     @property
-    def recurpattern(self):
+    def recurpattern(self) -> str:
         if 'RRULE' in self._vevents[self.ref]:
             return self._vevents[self.ref]['RRULE'].to_ical().decode('utf-8')
         else:
             return ''
 
     @property
-    def recurobject(self):
+    def recurobject(self) -> icalendar.vRecur:
         if 'RRULE' in self._vevents[self.ref]:
             return self._vevents[self.ref]['RRULE']
         else:
             return icalendar.vRecur()
 
-    def update_rrule(self, rrule):
+    def update_rrule(self, rrule: str) -> None:
         self._vevents['PROTO'].pop('RRULE')
         if rrule is not None:
             self._vevents['PROTO'].add('RRULE', rrule)
 
     @property
-    def recurrence_id(self):
+    def recurrence_id(self) -> Union[dt.datetime, str]:
         """return the "original" start date of this event (i.e. their recurrence-id)
         """
         if self.ref == 'PROTO':
@@ -259,7 +271,7 @@ class Event:
         else:
             return pytz.UTC.localize(dt.datetime.utcfromtimestamp(int(self.ref)))
 
-    def increment_sequence(self):
+    def increment_sequence(self) -> None:
         """update the SEQUENCE number, call before saving this event"""
         # TODO we might want to do this automatically in raw() everytime
         # the event has changed, this will f*ck up the tests though
@@ -269,7 +281,7 @@ class Event:
             self._vevents[self.ref]['SEQUENCE'] = 0
 
     @property
-    def symbol_strings(self):
+    def symbol_strings(self) -> Dict[str, str]:
         if self._locale['unicode_symbols']:
             return {
                 'recurring': '\N{Clockwise gapped circle arrow}',
@@ -277,7 +289,12 @@ class Event:
                 'range': '\N{Left right arrow}',
                 'range_end': '\N{Rightwards arrow to bar}',
                 'range_start': '\N{Rightwards arrow from bar}',
-                'right_arrow': '\N{Rightwards arrow}'
+                'right_arrow': '\N{Rightwards arrow}',
+                'cancelled': '\N{Cross mark}',
+                'confirmed': '\N{Heavy check mark}',
+                'tentative': '?',
+                'declined': '\N{Cross mark}',
+                'accepted': '\N{Heavy check mark}',
             }
         else:
             return {
@@ -286,43 +303,48 @@ class Event:
                 'range': '<->',
                 'range_end': '->|',
                 'range_start': '|->',
-                'right_arrow': '->'
+                'right_arrow': '->',
+                'cancelled': 'X',
+                'confirmed': 'V',
+                'tentative': '?',
+                'declined': 'X',
+                'accepted': 'V',
             }
 
     @property
-    def start_local(self):
+    def start_local(self) -> dt.datetime:
         """self.start() localized to local timezone"""
         return self.start
 
     @property
-    def end_local(self):
+    def end_local(self) -> dt.datetime:
         """self.end() localized to local timezone"""
         return self.end
 
     @property
-    def start(self):
+    def start(self) -> dt.datetime:
         """this should return the start date(time) as saved in the event"""
         return self._start
 
     @property
-    def end(self):
+    def end(self) -> dt.datetime:
         """this should return the end date(time) as saved in the event or
         implicitly defined by start and duration"""
         return self._end
 
     @property
-    def duration(self):
+    def duration(self) -> dt.timedelta:
         try:
             return self._vevents[self.ref]['DURATION'].dt
         except KeyError:
             return self.end - self.start
 
     @property
-    def uid(self):
+    def uid(self) -> str:
         return self._vevents[self.ref]['UID']
 
     @property
-    def organizer(self):
+    def organizer(self) -> str:
         if 'ORGANIZER' not in self._vevents[self.ref]:
             return ''
         organizer = self._vevents[self.ref]['ORGANIZER']
@@ -334,25 +356,20 @@ class Event:
             return email
 
     @property
-    def url(self):
+    def url(self) -> str:
         if 'URL' not in self._vevents[self.ref]:
             return ''
         return self._vevents[self.ref]['URL']
 
-    def update_url(self, url):
+    def update_url(self, url: str) -> None:
         if url:
             self._vevents[self.ref]['URL'] = url
         else:
             self._vevents[self.ref].pop('URL')
 
     @staticmethod
-    def _create_calendar():
-        """
-        create the calendar
-
-        :returns: calendar
-        :rtype: icalendar.Calendar()
-        """
+    def _create_calendar() -> icalendar.Calendar:
+        """create the calendar"""
         calendar = icalendar.Calendar()
         calendar.add('version', '2.0')
         calendar.add(
@@ -361,10 +378,8 @@ class Event:
         return calendar
 
     @property
-    def raw(self):
-        """needed for vdirsyncer compatibility
-
-        return text
+    def raw(self) -> str:
+        """Creates a VCALENDAR containing VTIMEZONEs
         """
         calendar = self._create_calendar()
         tzs = []
@@ -386,7 +401,7 @@ class Event:
             calendar.add_component(vevent)
         return calendar.to_ical().decode('utf-8')
 
-    def export_ics(self, path):
+    def export_ics(self, path: str) -> None:
         """export event as ICS
         """
         export_path = os.path.expanduser(path)
@@ -394,7 +409,7 @@ class Event:
             fh.write(self.raw)
 
     @property
-    def summary(self):
+    def summary(self) -> str:
         description = None
         date = self._vevents[self.ref].get('x-birthday', None)
         if date:
@@ -423,17 +438,15 @@ class Event:
                 suffix = 'rd'
             else:
                 suffix = 'th'
-            return '{name}\'s {number}{suffix} {desc}{leap}'.format(
-                name=name, number=number, suffix=suffix, desc=description, leap=leap,
-            )
+            return f'{name}\'s {number}{suffix} {description}{leap}'
         else:
             return self._vevents[self.ref].get('SUMMARY', '')
 
-    def update_summary(self, summary):
+    def update_summary(self, summary: str) -> None:
         self._vevents[self.ref]['SUMMARY'] = summary
 
     @staticmethod
-    def _can_handle_alarm(alarm):
+    def _can_handle_alarm(alarm) -> bool:
         """
         Decides whether we can handle a certain alarm.
         """
@@ -441,7 +454,7 @@ class Event:
             isinstance(alarm.get('TRIGGER').dt, dt.timedelta)
 
     @property
-    def alarms(self):
+    def alarms(self) -> List[Tuple[dt.timedelta, str]]:
         """
         Returns a list of all alarms in th original event that we can handle. Unknown types of
         alarms are ignored.
@@ -450,7 +463,7 @@ class Event:
                 for a in self._vevents[self.ref].subcomponents
                 if a.name == 'VALARM' and self._can_handle_alarm(a)]
 
-    def update_alarms(self, alarms):
+    def update_alarms(self, alarms: List[Tuple[dt.timedelta, str]]) -> None:
         """
         Replaces all alarms in the event that can be handled with the ones provided.
         """
@@ -468,24 +481,24 @@ class Event:
         self._vevents[self.ref].subcomponents = components
 
     @property
-    def location(self):
+    def location(self) -> str:
         return self._vevents[self.ref].get('LOCATION', '')
 
-    def update_location(self, location):
+    def update_location(self, location: str) -> None:
         if location:
             self._vevents[self.ref]['LOCATION'] = location
         else:
             self._vevents[self.ref].pop('LOCATION')
 
     @property
-    def attendees(self):
+    def attendees(self) -> str:
         addresses = self._vevents[self.ref].get('ATTENDEE', [])
         if not isinstance(addresses, list):
             addresses = [addresses, ]
         return ", ".join([address.split(':')[-1]
                           for address in addresses])
 
-    def update_attendees(self, attendees):
+    def update_attendees(self, attendees: List[str]):
         assert isinstance(attendees, list)
         attendees = [a.strip().lower() for a in attendees if a != ""]
         if len(attendees) > 0:
@@ -515,30 +528,31 @@ class Event:
             self._vevents[self.ref].pop('ATTENDEE')
 
     @property
-    def categories(self):
+    def categories(self) -> str:
         try:
             return self._vevents[self.ref].get('CATEGORIES', '').to_ical().decode('utf-8')
         except AttributeError:
             return ''
 
-    def update_categories(self, categories):
+    def update_categories(self, categories: List[str]) -> None:
         assert isinstance(categories, list)
+        categories = [c.strip() for c in categories if c != ""]
         self._vevents[self.ref].pop('CATEGORIES', False)
         if categories:
             self._vevents[self.ref].add('CATEGORIES', categories)
 
     @property
-    def description(self):
+    def description(self) -> str:
         return self._vevents[self.ref].get('DESCRIPTION', '')
 
-    def update_description(self, description):
+    def update_description(self, description: str):
         if description:
             self._vevents[self.ref]['DESCRIPTION'] = description
         else:
             self._vevents[self.ref].pop('DESCRIPTION')
 
     @property
-    def _recur_str(self):
+    def _recur_str(self) -> str:
         if self.recurring:
             recurstr = ' ' + self.symbol_strings['recurring']
         else:
@@ -546,24 +560,53 @@ class Event:
         return recurstr
 
     @property
-    def _alarm_str(self):
+    def _alarm_str(self) -> str:
         if self.alarms:
             alarmstr = ' ' + self.symbol_strings['alarming']
         else:
             alarmstr = ''
         return alarmstr
 
-    def format(self, format_string, relative_to, env=None, colors=True):
+    @property
+    def _status_str(self) -> str:
+        if self.status == 'CANCELLED':
+            statusstr = self.symbol_strings['cancelled']
+        elif self.status == 'TENTATIVE':
+            statusstr = self.symbol_strings['tentative']
+        elif self.status == 'CONFIRMED':
+            statusstr = self.symbol_strings['confirmed']
+        else:
+            statusstr = ''
+        return statusstr
+
+    @property
+    def _partstat_str(self) -> str:
+        partstat = self.partstat
+        if partstat == 'ACCEPTED':
+            partstatstr = self.symbol_strings['accepted']
+        elif partstat == 'TENTATIVE':
+            partstatstr = self.symbol_strings['tentative']
+        elif partstat == 'DECLINED':
+            partstatstr = self.symbol_strings['declined']
+        else:
+            partstatstr = ''
+        return partstatstr
+
+    def attributes(
+            self,
+            relative_to: Union[Tuple[dt.date, dt.date], dt.date],
+            env=None,
+            colors: bool=True,
+    ):
         """
         :param colors: determines if colors codes should be printed or not
-        :type colors: bool
         """
         env = env or {}
 
         attributes = {}
-        try:
+        if isinstance(relative_to, tuple):
             relative_to_start, relative_to_end = relative_to
-        except TypeError:
+        else:
             relative_to_start = relative_to_end = relative_to
 
         if isinstance(relative_to_end, dt.datetime):
@@ -679,22 +722,34 @@ class Event:
         attributes["repeat-symbol"] = self._recur_str
         attributes["repeat-pattern"] = self.recurpattern
         attributes["alarm-symbol"] = self._alarm_str
+        attributes["status-symbol"] = self._status_str
+        attributes["partstat-symbol"] = self._partstat_str
         attributes["title"] = self.summary
         attributes["organizer"] = self.organizer.strip()
-        attributes["description"] = self.description.strip()
+
+        formatters = FORMATTERS.values()
+        if len(formatters) == 1:
+            fmt: Callable[[str], str] = list(formatters)[0]
+        else:
+            def fmt(s: str) -> str: return s.strip()
+
+        attributes["description"] = fmt(self.description)
         attributes["description-separator"] = ""
         if attributes["description"]:
             attributes["description-separator"] = " :: "
         attributes["location"] = self.location.strip()
         attributes["attendees"] = self.attendees
-        attributes["all-day"] = allday
+        attributes["all-day"] = str(allday)
         attributes["categories"] = self.categories
         attributes['uid'] = self.uid
         attributes['url'] = self.url
+        attributes['url-separator'] = ""
+        if attributes['url']:
+            attributes['url-separator'] = " :: "
 
         if "calendars" in env and self.calendar in env["calendars"]:
             cal = env["calendars"][self.calendar]
-            attributes["calendar-color"] = get_color(cal.get('color', ''))
+            attributes["calendar-color"] = cal.get('color', '')
             attributes["calendar"] = cal.get("displayname", self.calendar)
         else:
             attributes["calendar-color"] = attributes["calendar"] = ''
@@ -717,25 +772,26 @@ class Event:
 
         attributes['status'] = self.status + ' ' if self.status else ''
         attributes['cancelled'] = 'CANCELLED ' if self.status == 'CANCELLED' else ''
-        return format_string.format(**dict(attributes)) + attributes["reset"]
+        return attributes
 
-    def duplicate(self):
-        """duplicate this event's PROTO event
-
-        :rtype: Event
-        """
+    def duplicate(self) -> 'Event':
+        """duplicate this event's PROTO event"""
         new_uid = generate_random_uid()
         vevent = self._vevents['PROTO'].copy()
         vevent['SEQUENCE'] = 0
         vevent['UID'] = icalendar.vText(new_uid)
         vevent['SUMMARY'] = icalendar.vText(vevent['SUMMARY'] + ' Copy')
-        event = self.fromVEvents([vevent])
+        event = self.fromVEvents([vevent], locale=self._locale)
         event.calendar = self.calendar
         event._locale = self._locale
         return event
 
-    def delete_instance(self, instance):
-        """delete an instance from this event"""
+    def delete_instance(self, instance: dt.datetime) -> None:
+        """delete an instance from this event
+
+        we don't check, if that instance is an instance of the recurrence rules
+        defined in the event
+        """
         assert self.recurring
         delete_instance(self._vevents['PROTO'], instance)
 
@@ -754,9 +810,17 @@ class Event:
             self._vevents.pop(key)
 
     @property
-    def status(self):
+    def status(self) -> str:
         return self._vevents[self.ref].get('STATUS', '')
 
+    @property
+    def partstat(self) -> Optional[str]:
+        for attendee in self._vevents[self.ref].get('ATTENDEE', []):
+            for address in self.addresses:
+                if attendee == 'mailto:' + address:
+                    return attendee.params.get('PARTSTAT', '')
+        return None
+
 
 class DatetimeEvent(Event):
     pass
@@ -766,7 +830,8 @@ class LocalizedEvent(DatetimeEvent):
     """
     see parent
     """
-    def __init__(self, *args, **kwargs):
+
+    def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         try:
             starttz = getattr(self._vevents[self.ref]['DTSTART'].dt, 'tzinfo', None)
@@ -801,14 +866,14 @@ class LocalizedEvent(DatetimeEvent):
             self._end = endtz.localize(self._end)
 
     @property
-    def start_local(self):
+    def start_local(self) -> dt.datetime:
         """
         see parent
         """
         return self.start.astimezone(self._locale['local_timezone'])
 
     @property
-    def end_local(self):
+    def end_local(self) -> dt.datetime:
         """
         see parent
         """
@@ -818,22 +883,22 @@ class LocalizedEvent(DatetimeEvent):
 class FloatingEvent(DatetimeEvent):
     """
     """
-    allday = False
+    allday: bool = False
 
     @property
-    def start_local(self):
+    def start_local(self) -> dt.datetime:
         return self._locale['local_timezone'].localize(self.start)
 
     @property
-    def end_local(self):
+    def end_local(self) -> dt.datetime:
         return self._locale['local_timezone'].localize(self.end)
 
 
 class AllDayEvent(Event):
-    allday = True
+    allday: bool = True
 
     @property
-    def end(self):
+    def end(self) -> dt.datetime:
         end = super().end
         if end == self.start:
             # https://github.com/pimutils/khal/issues/129
@@ -846,27 +911,28 @@ class AllDayEvent(Event):
         return end - dt.timedelta(days=1)
 
     @property
-    def duration(self):
+    def duration(self) -> dt.timedelta:
         try:
             return self._vevents[self.ref]['DURATION'].dt
         except KeyError:
             return self.end - self.start + dt.timedelta(days=1)
 
 
-def create_timezone(tz, first_date=None, last_date=None):
+def create_timezone(
+    tz: pytz.BaseTzInfo,
+    first_date: Optional[dt.datetime]=None,
+    last_date: Optional[dt.datetime]=None
+) -> icalendar.Timezone:
     """
     create an icalendar vtimezone from a pytz.tzinfo object
 
     :param tz: the timezone
-    :type tz: pytz.tzinfo
     :param first_date: the very first datetime that needs to be included in the
     transition times, typically the DTSTART value of the (first recurring)
     event
-    :type first_date: datetime.datetime
     :param last_date: the last datetime that needs to included, typically the
     end of the (very last) event (of a recursion set)
     :returns: timezone information
-    :rtype: icalendar.Timezone()
 
     we currently have a problem here:
 
@@ -882,8 +948,7 @@ def create_timezone(tz, first_date=None, last_date=None):
     easy solution, we'd really need to ship another version of the OLSON DB.
 
     """
-
-    if isinstance(tz, pytz.tzinfo.StaticTzInfo):
+    if isinstance(tz, StaticTzInfo):
         return _create_timezone_static(tz)
 
     # TODO last_date = None, recurring to infinity
@@ -895,18 +960,18 @@ def create_timezone(tz, first_date=None, last_date=None):
 
     dst = {
         one[2]: 'DST' in two.__repr__()
-        for one, two in iter(tz._tzinfos.items())
+        for one, two in iter(tz._tzinfos.items())  # type: ignore
     }
     bst = {
         one[2]: 'BST' in two.__repr__()
-        for one, two in iter(tz._tzinfos.items())
+        for one, two in iter(tz._tzinfos.items())  # type: ignore
     }
 
     # looking for the first and last transition time we need to include
-    first_num, last_num = 0, len(tz._utc_transition_times) - 1
-    first_tt = tz._utc_transition_times[0]
-    last_tt = tz._utc_transition_times[-1]
-    for num, transtime in enumerate(tz._utc_transition_times):
+    first_num, last_num = 0, len(tz._utc_transition_times) - 1  # type: ignore
+    first_tt = tz._utc_transition_times[0]  # type: ignore
+    last_tt = tz._utc_transition_times[-1]  # type: ignore
+    for num, transtime in enumerate(tz._utc_transition_times):  # type: ignore
         if first_date > transtime > first_tt:
             first_num = num
             first_tt = transtime
@@ -914,11 +979,11 @@ def create_timezone(tz, first_date=None, last_date=None):
             last_num = num
             last_tt = transtime
 
-    timezones = {}
+    timezones: Dict[str, icalendar.Component] = {}
     for num in range(first_num, last_num + 1):
-        name = tz._transition_info[num][2]
+        name = tz._transition_info[num][2]  # type: ignore
         if name in timezones:
-            ttime = tz.fromutc(tz._utc_transition_times[num]).replace(tzinfo=None)
+            ttime = tz.fromutc(tz._utc_transition_times[num]).replace(tzinfo=None)  # type: ignore
             if 'RDATE' in timezones[name]:
                 timezones[name]['RDATE'].dts.append(
                     icalendar.prop.vDDDTypes(ttime))
@@ -931,12 +996,12 @@ def create_timezone(tz, first_date=None, last_date=None):
         else:
             subcomp = icalendar.TimezoneStandard()
 
-        subcomp.add('TZNAME', tz._transition_info[num][2])
+        subcomp.add('TZNAME', tz._transition_info[num][2])  # type: ignore
         subcomp.add(
             'DTSTART',
-            tz.fromutc(tz._utc_transition_times[num]).replace(tzinfo=None))
-        subcomp.add('TZOFFSETTO', tz._transition_info[num][0])
-        subcomp.add('TZOFFSETFROM', tz._transition_info[num - 1][0])
+            tz.fromutc(tz._utc_transition_times[num]).replace(tzinfo=None))  # type: ignore
+        subcomp.add('TZOFFSETTO', tz._transition_info[num][0])  # type: ignore
+        subcomp.add('TZOFFSETFROM', tz._transition_info[num - 1][0])  # type: ignore
         timezones[name] = subcomp
 
     for subcomp in timezones.values():
@@ -945,13 +1010,11 @@ def create_timezone(tz, first_date=None, last_date=None):
     return timezone
 
 
-def _create_timezone_static(tz):
-    """create an icalendar vtimezone from a pytz.tzinfo.StaticTzInfo
+def _create_timezone_static(tz: StaticTzInfo) -> icalendar.Timezone:
+    """create an icalendar vtimezone from a StaticTzInfo
 
     :param tz: the timezone
-    :type tz: pytz.tzinfo.StaticTzInfo
     :returns: timezone information
-    :rtype: icalendar.Timezone()
     """
     timezone = icalendar.Timezone()
     timezone.add('TZID', tz)
@@ -959,7 +1022,7 @@ def _create_timezone_static(tz):
     subcomp.add('TZNAME', tz)
     subcomp.add('DTSTART', dt.datetime(1601, 1, 1))
     subcomp.add('RDATE', dt.datetime(1601, 1, 1))
-    subcomp.add('TZOFFSETTO', tz._utcoffset)
-    subcomp.add('TZOFFSETFROM', tz._utcoffset)
+    subcomp.add('TZOFFSETTO', tz._utcoffset)  # type: ignore
+    subcomp.add('TZOFFSETFROM', tz._utcoffset)  # type: ignore
     timezone.add_component(subcomp)
     return timezone
