@@ -30,37 +30,38 @@ import itertools
 import logging
 import os
 import os.path
-from typing import Any, Dict, Iterable, List, Optional, Tuple, Union  # noqa
+from typing import Any, Dict, Iterable, List, Optional, Set, Tuple, Union  # noqa
 
+from ..custom_types import CalendarConfiguration, EventCreationTypes, LocaleConfiguration
+from ..icalendar import new_vevent
 from . import backend
 from .event import Event
-from .exceptions import (CouldNotCreateDbDir, DuplicateUid, NonUniqueUID,
-                         ReadOnlyCalendarError, UnsupportedFeatureError,
-                         UpdateFailed)
-from .vdir import (AlreadyExistingError, CollectionNotFoundError, Vdir,
-                   get_etag_from_file)
+from .exceptions import (
+    DuplicateUid,
+    EtagMissmatch,
+    NonUniqueUID,
+    ReadOnlyCalendarError,
+    UnsupportedFeatureError,
+    UpdateFailed,
+)
+from .vdir import (
+    AlreadyExistingError,
+    CollectionNotFoundError,
+    Vdir,
+    WrongEtagError,
+    get_etag_from_file,
+)
 
 logger = logging.getLogger('khal')
 
 
-def create_directory(path: str):
-    if not os.path.isdir(path):
-        if os.path.exists(path):
-            raise RuntimeError(f'{path} is not a directory.')
-        try:
-            os.makedirs(path, mode=0o750)
-        except OSError as error:
-            logger.critical(f'failed to create {path}: {error}')
-            raise CouldNotCreateDbDir()
-
-
 class CalendarCollection:
     """CalendarCollection allows access to various calendars stored in vdirs
 
     all calendars are cached in an sqlitedb for performance reasons"""
 
     def __init__(self,
-                 calendars=None,
+                 calendars: Dict[str, CalendarConfiguration],
                  hmethod: str='fg',
                  default_color: str='',
                  multiple: str='',
@@ -68,15 +69,18 @@ class CalendarCollection:
                  color: str='',
                  priority: int=10,
                  highlight_event_days: bool=False,
-                 locale: Optional[Dict[str, Any]]=None,
+                 locale: Optional[LocaleConfiguration]=None,
                  dbpath: Optional[str]=None,
                  ) -> None:
-        locale = locale or {}
+        assert locale
         assert dbpath is not None
         assert calendars is not None
-        self._calendars = calendars
-        self._default_calendar_name = None  # type: Optional[str]
-        self._storages = {}  # type: Dict[str, Vdir]
+
+        self._calendars: Dict[str, CalendarConfiguration] = calendars
+        self._default_calendar_name: Optional[str] = None
+        self._storages: Dict[str, Vdir] = {}
+        file_ext: str
+
         for name, calendar in self._calendars.items():
             ctype = calendar.get('ctype', 'calendar')
             if ctype == 'calendar':
@@ -101,7 +105,7 @@ class CalendarCollection:
         self.highlight_event_days = highlight_event_days
         self._locale = locale
         self._backend = backend.SQLiteDb(self.names, dbpath, self._locale)
-        self._last_ctags = {}  # type: Dict[str, str]
+        self._last_ctags: Dict[str, str] = {}
         self.update_db()
 
     @property
@@ -109,7 +113,7 @@ class CalendarCollection:
         return [c for c in self._calendars if not self._calendars[c].get('readonly', False)]
 
     @property
-    def calendars(self) -> Iterable[str]:
+    def calendars(self) -> Iterable[CalendarConfiguration]:
         return self._calendars.values()
 
     @property
@@ -121,7 +125,7 @@ class CalendarCollection:
         return self._default_calendar_name
 
     @default_calendar_name.setter
-    def default_calendar_name(self, default: str):
+    def default_calendar_name(self, default: str) -> None:
         if default is None:
             self._default_calendar_name = default
         elif default not in self.names:
@@ -165,9 +169,12 @@ class CalendarCollection:
         )
         return list(set(calendars))
 
-    def update(self, event: Event):
+    def update(self, event: Event) -> None:
         """update `event` in vdir and db"""
-        assert event.etag
+        assert event.etag is not None
+        assert event.calendar is not None
+        assert event.href is not None
+        assert event.raw is not None
         if self._calendars[event.calendar]['readonly']:
             raise ReadOnlyCalendarError()
         with self._backend.at_once():
@@ -175,9 +182,11 @@ class CalendarCollection:
             self._backend.update(event.raw, event.href, event.etag, calendar=event.calendar)
             self._backend.set_ctag(self._local_ctag(event.calendar), calendar=event.calendar)
 
-    def force_update(self, event: Event, collection: Optional[str]=None):
+    def force_update(self, event: Event, collection: Optional[str]=None) -> None:
         """update `event` even if an event with the same uid/href already exists"""
+        href: str
         calendar = collection if collection is not None else event.calendar
+        assert calendar is not None
         if self._calendars[calendar]['readonly']:
             raise ReadOnlyCalendarError()
 
@@ -191,21 +200,26 @@ class CalendarCollection:
             self._backend.update(event.raw, href, etag, calendar=calendar)
             self._backend.set_ctag(self._local_ctag(calendar), calendar=calendar)
 
-    def new(self, event: Event, collection: Optional[str]=None):
-        """save a new event to the vdir and the database
+    def insert(self, event: Event, collection: Optional[str]=None) -> None:
+        """Insert a new event to the vdir and the database
 
-        param event: the event that should be updated, will get a new href and
-            etag properties
-        type event: event.Event
+        The event will get a new href and etag properties. If ``collection`` is
+        ``None``, then ``event.calendar`` must be defined.
+
+        :param event: the event to be inserted.
         """
+        # TODO FIXME not all `event`s are actually of type Event, we also uptade
+        # with vdir.Items. Those don't have an .href or .etag property which we
+        # than attach anyway. Works, but pretty ugly and any type checker will
+        # complain.
         calendar = collection if collection is not None else event.calendar
+        assert calendar is not None
         if hasattr(event, 'etag'):
             assert not event.etag
         if self._calendars[calendar]['readonly']:
             raise ReadOnlyCalendarError()
 
         with self._backend.at_once():
-
             try:
                 event.href, event.etag = self._storages[calendar].upload(event)
             except AlreadyExistingError as Error:
@@ -214,27 +228,51 @@ class CalendarCollection:
             self._backend.update(event.raw, event.href, event.etag, calendar=calendar)
             self._backend.set_ctag(self._local_ctag(calendar), calendar=calendar)
 
-    def delete(self, href: str, etag: str, calendar: str):
+    def delete(self, href: str, etag: Optional[str], calendar: str) -> None:
+        """Delete an event specified by `href` from `calendar`"""
         if self._calendars[calendar]['readonly']:
             raise ReadOnlyCalendarError()
-        self._storages[calendar].delete(href, etag)
+        try:
+            self._storages[calendar].delete(href, etag)
+        except WrongEtagError:
+            raise EtagMissmatch()
         self._backend.delete(href, calendar=calendar)
 
+    def delete_instance(self,
+                        href: str,
+                        etag: Optional[str],
+                        calendar: str,
+                        rec_id: dt.datetime,
+                        ) -> Event:
+        """Delete a recurrence instance from an event specified by `href` from `calendar`
+
+        returns the updated event
+        """
+        if self._calendars[calendar]['readonly']:
+            raise ReadOnlyCalendarError()
+        event = self.get_event(href, calendar)
+        if etag and etag != event.etag:
+            raise EtagMissmatch()
+
+        event.delete_instance(rec_id)
+        self.update(event)
+        return event
+
     def get_event(self, href: str, calendar: str) -> Event:
         """get an event by its href from the datatbase"""
-        return self._construct_event(
-            self._backend.get(href, calendar), href=href, calendar=calendar,
-        )
+        event_str, etag = self._backend.get_with_etag(href, calendar)
+        return self._construct_event(event_str, etag=etag, href=href, calendar=calendar)
 
     def _construct_event(self,
                          item: str,
                          href: str,
-                         start: dt.datetime = None,
-                         end: dt.datetime = None,
+                         start: Optional[Union[dt.datetime, dt.date]] = None,
+                         end: Optional[Union[dt.datetime, dt.date]] = None,
                          ref: str='PROTO',
-                         etag: str=None,
-                         calendar: str=None,
+                         etag: Optional[str]=None,
+                         calendar: Optional[str]=None,
                          ) -> Event:
+        assert calendar is not None
         event = Event.fromString(
             item,
             locale=self._locale,
@@ -246,22 +284,42 @@ class CalendarCollection:
             ref=ref,
             color=self._calendars[calendar]['color'],
             readonly=self._calendars[calendar]['readonly'],
+            addresses=self._calendars[calendar]['addresses'],
         )
         return event
 
-    def change_collection(self, event: Event, new_collection: str):
+    def change_collection(self, event: Event, new_collection: str) -> None:
+        """Moves `event` to a new collection (calendar)"""
         href, etag, calendar = event.href, event.etag, event.calendar
         event.etag = None
-        self.new(event, new_collection)
+        self.insert(event, new_collection)
+        assert href is not None
+        assert calendar is not None
         self.delete(href, etag, calendar=calendar)
 
-    def new_event(self, ical: str, collection: str):
-        """creates and returns (but does not insert) new event from ical
+    def create_event_from_ics(self,
+                              ical: str,
+                              calendar_name: str,
+                              etag: Optional[str]=None,
+                              href: Optional[str]=None,
+                              ) -> Event:
+        """creates and returns (but does not insert) a new event from ical
         string"""
-        calendar = collection or self.writable_names[0]
-        return Event.fromString(ical, locale=self._locale, calendar=calendar)
+        calendar = calendar_name or self.writable_names[0]
+        return Event.fromString(ical, locale=self._locale, calendar=calendar, etag=etag, href=href)
 
-    def update_db(self):
+    def create_event_from_dict(self,
+                               event_dict: EventCreationTypes,
+                               calendar_name: Optional[str] = None,
+                               ) -> Event:
+        """Creates an Event from the method's arguments
+        """
+        vevent = new_vevent(locale=self._locale, **event_dict)
+        calendar_name = calendar_name or self.default_calendar_name or self.writable_names[0]
+        assert calendar_name is not None
+        return self.create_event_from_ics(vevent.to_ical(), calendar_name)
+
+    def update_db(self) -> None:
         """update the db from the vdir,
 
         should be called after every change to the vdir
@@ -303,11 +361,11 @@ class CalendarCollection:
             self._last_ctags[calendar] = local_ctag
         return local_ctag != self._backend.get_ctag(calendar)
 
-    def _db_update(self, calendar: str):
+    def _db_update(self, calendar: str) -> None:
         """implements the actual db update on a per calendar base"""
         local_ctag = self._local_ctag(calendar)
         db_hrefs = {href for href, etag in self._backend.list(calendar)}
-        storage_hrefs = set()
+        storage_hrefs: Set[str] = set()
         bdays = self._calendars[calendar].get('ctype') == 'birthdays'
 
         with self._backend.at_once():
@@ -360,11 +418,11 @@ class CalendarCollection:
             return 'highlight_days_color'
         if len(calendars) == 1:
             return 'calendar ' + calendars[0]
-        if self.multiple != '' and not(self.multiple_on_overflow and len(calendars) == 2):
+        if self.multiple != '' and not (self.multiple_on_overflow and len(calendars) == 2):
             return 'highlight_days_multiple'
         return ('calendar ' + calendars[0], 'calendar ' + calendars[1])
 
-    def get_styles(self, date: dt.date, focus: bool) -> Union[str, None, Tuple[str, str]]:
+    def get_styles(self, date: dt.date, focus: bool) -> Optional[Union[str, Tuple[str, str]]]:
         if focus:
             if date == date.today():
                 return 'today focus'
