@@ -24,8 +24,9 @@ import datetime as dt
 import glob
 import logging
 import os
+import pathlib
 from os.path import expanduser, expandvars, join
-from typing import Optional
+from typing import Callable, Iterable, Literal, Optional, Union
 
 import pytz
 import xdg
@@ -61,20 +62,18 @@ def is_timezone(tzstring: Optional[str]) -> dt.tzinfo:
         raise VdtValueError(f"Unknown timezone {tzstring}")
 
 
-def is_timedelta(string):
+def is_timedelta(string: str) -> dt.timedelta:
     try:
         return guesstimedeltafstr(string)
     except ValueError:
         raise VdtValueError(f"Invalid timedelta: {string}")
 
 
-def weeknumber_option(option):
+def weeknumber_option(option: str) -> Union[Literal['left', 'right'], Literal[False]]:
     """checks if *option* is a valid value
 
     :param option: the option the user set in the config file
-    :type option: str
-    :returns: off, left, right
-    :rtype: str/bool
+    :returns: 'off', 'left', 'right' or False
     """
     option = option.lower()
     if option == 'left':
@@ -89,13 +88,10 @@ def weeknumber_option(option):
             "'off', 'left' or 'right'")
 
 
-def monthdisplay_option(option):
+def monthdisplay_option(option: str) -> Literal['firstday', 'firstfullweek']:
     """checks if *option* is a valid value
 
     :param option: the option the user set in the config file
-    :type option: str
-    :returns: firstday, firstfullweek
-    :rtype: str/bool
     """
     option = option.lower()
     if option == 'firstday':
@@ -109,19 +105,19 @@ def monthdisplay_option(option):
         )
 
 
-def expand_path(path):
+def expand_path(path: str) -> str:
     """expands `~` as well as variable names"""
     return expanduser(expandvars(path))
 
 
-def expand_db_path(path):
+def expand_db_path(path: str) -> str:
     """expands `~` as well as variable names, defaults to $XDG_DATA_HOME"""
     if path is None:
         path = join(xdg.BaseDirectory.xdg_data_home, 'khal', 'khal.db')
     return expanduser(expandvars(path))
 
 
-def is_color(color):
+def is_color(color: str) -> str:
     """checks if color represents a valid color
 
     raises a VdtValueError if color is not valid
@@ -141,7 +137,7 @@ def is_color(color):
     raise VdtValueError(color)
 
 
-def test_default_calendar(config):
+def test_default_calendar(config) -> None:
     """test if config['default']['default_calendar'] is set to a sensible
     value
     """
@@ -159,7 +155,7 @@ def test_default_calendar(config):
         raise InvalidSettingsError()
 
 
-def get_color_from_vdir(path):
+def get_color_from_vdir(path: str) -> Optional[str]:
     try:
         color = Vdir(path, '.ics').get_meta('color')
     except CollectionNotFoundError:
@@ -176,7 +172,7 @@ def get_color_from_vdir(path):
     return color
 
 
-def get_unique_name(path, names):
+def get_unique_name(path: str, names: Iterable[str]) -> str:
     # TODO take care of edge cases, make unique name finding less brain-dead
     try:
         name = Vdir(path, '.ics').get_meta('displayname')
@@ -192,23 +188,50 @@ def get_unique_name(path, names):
     return name
 
 
-def get_all_vdirs(path):
+def get_all_vdirs(expand_path: str) -> Iterable[str]:
     """returns a list of paths, expanded using glob
     """
-    items = glob.glob(path)
-    return items
+    # FIXME currently returns a list of all directories in path
+    # we add an additional / at the end to make sure we are only getting
+    # directories
+    items = glob.glob(f'{expand_path}/', recursive=True)
+    paths = [pathlib.Path(item) for item in sorted(items, key=len, reverse=True)]
+    leaves = set()
+    parents = set()
+    for path in paths:
+        if path in parents:
+            # we have already seen the current directory as the parent of
+            # another directory, so this directory can't be a vdir
+            continue
+        parents.add(path.parent)
+        leaves.add(path)
+    # sort to make sure that auto generated names are always identical
+    return sorted(os.fspath(path) for path in leaves)
 
 
-def get_vdir_type(_):
+def get_vdir_type(_: str) -> str:
     # TODO implement
     return 'calendar'
 
+def validate_palette_entry(attr, definition: str) -> bool:
+    if len(definition) not in (2, 3, 5):
+        logging.error('Invalid color definition for %s: %s, must be of length, 2, 3, or 5',
+                      attr, definition)
+        return False
+    if (definition[0] not in COLORS and definition[0] != '') or \
+            (definition[1] not in COLORS and definition[1] != ''):
+        logging.error('Invalid color definition for %s: %s, must be one of %s',
+                      attr, definition, COLORS.keys())
+        return False
+    return True
 
 def config_checks(
-        config,
-        _get_color_from_vdir=get_color_from_vdir,
-        _get_vdir_type=get_vdir_type):
+    config,
+    _get_color_from_vdir: Callable=get_color_from_vdir,
+    _get_vdir_type: Callable=get_vdir_type,
+) -> None:
     """do some tests on the config we cannot do with configobj's validator"""
+    # TODO rename or split up, we are also expanding vdirs of type discover
     if len(config['calendars'].keys()) < 1:
         logger.fatal('Found no calendar section in the config file')
         raise InvalidSettingsError()
@@ -221,38 +244,26 @@ def config_checks(
             config['locale']['local_timezone'])
 
     # expand calendars with type = discover
-    vdirs_complete = []
-    vdir_colors_from_config = {}
-    for calendar in list(config['calendars'].keys()):
-        if not isinstance(config['calendars'][calendar], dict):
+    # we need a copy of config['calendars'], because we modify config in the body of the loop
+    for cname, cconfig in sorted(config['calendars'].items()):
+        if not isinstance(config['calendars'][cname], dict):
             logger.fatal('Invalid config file, probably missing calendar sections')
             raise InvalidSettingsError
-        if config['calendars'][calendar]['type'] == 'discover':
-            logger.debug(
-                f"discovering calendars in {config['calendars'][calendar]['path']}"
-            )
-            vdirs = get_all_vdirs(config['calendars'][calendar]['path'])
-            vdirs_complete += vdirs
-            if 'color' in config['calendars'][calendar]:
-                for vdir in vdirs:
-                    vdir_colors_from_config[vdir] = config['calendars'][calendar]['color']
-            config['calendars'].pop(calendar)
-    for vdir in sorted(vdirs_complete):
-        calendar = {'path': vdir,
-                    'color': _get_color_from_vdir(vdir),
+        if config['calendars'][cname]['type'] == 'discover':
+            logger.debug(f"discovering calendars in {cconfig['path']}")
+            vdirs_discovered = get_all_vdirs(cconfig['path'])
+            logger.debug(f"found the following vdirs: {vdirs_discovered}")
+            for vdir in vdirs_discovered:
+                vdir_config = {
+                    'path': vdir,
+                    'color': _get_color_from_vdir(vdir) or cconfig.get('color', None),
                     'type': _get_vdir_type(vdir),
-                    'readonly': False,
+                    'readonly': cconfig.get('readonly', False),
                     'priority': 10,
-                    }
-
-        # get color from config if not defined in vdir
-
-        if calendar['color'] is None and vdir in vdir_colors_from_config:
-            logger.debug(f"using collection's color for {vdir}")
-            calendar['color'] = vdir_colors_from_config[vdir]
-
-        name = get_unique_name(vdir, config['calendars'].keys())
-        config['calendars'][name] = calendar
+                }
+                unique_vdir_name = get_unique_name(vdir, config['calendars'].keys())
+                config['calendars'][unique_vdir_name] = vdir_config
+            config['calendars'].pop(cname)
 
     test_default_calendar(config)
     for calendar in config['calendars']:
@@ -261,3 +272,11 @@ def config_checks(
         if config['calendars'][calendar]['color'] == 'auto':
             config['calendars'][calendar]['color'] = \
                 _get_color_from_vdir(config['calendars'][calendar]['path'])
+
+    # check palette settings
+    valid_palette = True
+    for attr in config.get('palette', []):
+        valid_palette = valid_palette and validate_palette_entry(attr, config['palette'][attr])
+    if not valid_palette:
+        logger.fatal('Invalid palette entry')
+        raise InvalidSettingsError()
