@@ -21,36 +21,44 @@
 #
 """all functions related to terminal display are collected here"""
 
-from collections import namedtuple
 from itertools import zip_longest
+from typing import Dict, List, NamedTuple, Optional
+
+
+class NamedColor(NamedTuple):
+    color_index: int
+    light: bool
 
-NamedColor = namedtuple('NamedColor', ['index', 'light'])
 
 RTEXT = '\x1b[7m'  # reverse
 NTEXT = '\x1b[0m'  # normal
 BTEXT = '\x1b[1m'  # bold
 RESET = '\33[0m'
-COLORS = {
-    'black': NamedColor(index=0, light=False),
-    'dark red': NamedColor(index=1, light=False),
-    'dark green': NamedColor(index=2, light=False),
-    'brown': NamedColor(index=3, light=False),
-    'dark blue': NamedColor(index=4, light=False),
-    'dark magenta': NamedColor(index=5, light=False),
-    'dark cyan': NamedColor(index=6, light=False),
-    'white': NamedColor(index=7, light=False),
-    'light gray': NamedColor(index=7, light=True),
-    'dark gray': NamedColor(index=0, light=True),  # actually light black
-    'light red': NamedColor(index=1, light=True),
-    'light green': NamedColor(index=2, light=True),
-    'yellow': NamedColor(index=3, light=True),
-    'light blue': NamedColor(index=4, light=True),
-    'light magenta': NamedColor(index=5, light=True),
-    'light cyan': NamedColor(index=6, light=True)
+COLORS: Dict[str, NamedColor] = {
+    'black': NamedColor(color_index=0, light=False),
+    'dark red': NamedColor(color_index=1, light=False),
+    'dark green': NamedColor(color_index=2, light=False),
+    'brown': NamedColor(color_index=3, light=False),
+    'dark blue': NamedColor(color_index=4, light=False),
+    'dark magenta': NamedColor(color_index=5, light=False),
+    'dark cyan': NamedColor(color_index=6, light=False),
+    'white': NamedColor(color_index=7, light=False),
+    'light gray': NamedColor(color_index=7, light=True),
+    'dark gray': NamedColor(color_index=0, light=True),  # actually light black
+    'light red': NamedColor(color_index=1, light=True),
+    'light green': NamedColor(color_index=2, light=True),
+    'yellow': NamedColor(color_index=3, light=True),
+    'light blue': NamedColor(color_index=4, light=True),
+    'light magenta': NamedColor(color_index=5, light=True),
+    'light cyan': NamedColor(color_index=6, light=True)
 }
 
 
-def get_color(fg=None, bg=None, bold_for_light_color=False):
+def get_color(
+    fg: Optional[str]=None,
+    bg: Optional[str]=None,
+    bold_for_light_color: bool=False,
+) -> str:
     """convert foreground and/or background color in ANSI color codes
 
     colors can be a color name from the ANSI color palette (e.g. 'dark green'),
@@ -58,11 +66,8 @@ def get_color(fg=None, bg=None, bold_for_light_color=False):
     the style `#00FF00` or `#ABC`
 
     :param fg: foreground color
-    :type fg: str
     :param bg: background color
-    :type bg: str
     :returns: ANSI color code
-    :rtype: str
     """
 
     result = ''
@@ -73,7 +78,7 @@ def get_color(fg=None, bg=None, bold_for_light_color=False):
                 # 16 color palette
                 if not is_bg:
                     # foreground color
-                    c = 30 + COLORS[colorstring].index
+                    c = 30 + COLORS[colorstring].color_index
                     if COLORS[colorstring].light:
                         if bold_for_light_color:
                             color += '1;'
@@ -81,7 +86,7 @@ def get_color(fg=None, bg=None, bold_for_light_color=False):
                             c += 60
                 else:
                     # background color
-                    c = 40 + COLORS[colorstring].index
+                    c = 40 + COLORS[colorstring].color_index
                     if COLORS[colorstring].light:
                         if not bold_for_light_color:
                             c += 60
@@ -113,14 +118,17 @@ def get_color(fg=None, bg=None, bold_for_light_color=False):
     return result
 
 
-def colored(string, fg=None, bg=None, bold_for_light_color=True):
+def colored(
+    string: str,
+    fg: Optional[str]=None,
+    bg: Optional[str]=None,
+    bold_for_light_color: bool=True,
+) -> str:
     """colorize `string` with ANSI color codes
 
      see get_color for description of `fg`, `bg` and `bold_for_light_color`
     :param string: string to be colorized
-    :type string: str
     :returns: colorized string
-    :rtype: str
     """
     result = get_color(fg, bg, bold_for_light_color)
     result += string
@@ -129,7 +137,7 @@ def colored(string, fg=None, bg=None, bold_for_light_color=True):
     return result
 
 
-def merge_columns(lcolumn, rcolumn, width=25):
+def merge_columns(lcolumn: List[str], rcolumn: List[str], width: int=25) -> List[str]:
     """merge two lists elementwise together
 
     Wrap right columns to terminal width.
@@ -138,11 +146,8 @@ def merge_columns(lcolumn, rcolumn, width=25):
     out its (real) width automatically since it might contain ANSI
     escape sequences.
     """
-
     missing = len(rcolumn) - len(lcolumn)
     if missing > 0:
         lcolumn = lcolumn + missing * [width * ' ']
 
-    rows = ['    '.join(one) for one in zip_longest(
-        lcolumn, rcolumn, fillvalue='')]
-    return rows
+    return ['    '.join(one) for one in zip_longest(lcolumn, rcolumn, fillvalue='')]
