@@ -690,14 +690,14 @@ class TestSpecial:
         vevent = _get_vevent(_get_text('event_dt_rrule_until_before_start'))
         dtstart = icalendar_helpers.expand(vevent, berlin)
         # TODO test for logging message
-        assert dtstart is False
+        assert dtstart is None
 
     def test_event_invalid_rrule(self):
         """test handling if an event with RRULE will never occur"""
         vevent = _get_vevent(_get_text('event_rrule_no_occurence'))
         dtstart = icalendar_helpers.expand(vevent, berlin)
         # TODO test for logging message
-        assert dtstart is False
+        assert dtstart is None
 
 
 simple_rdate = """BEGIN:VEVENT
@@ -807,7 +807,7 @@ class TestSanitize:
         assert vevent['DTEND'].dt - vevent['DTSTART'].dt == dt.timedelta(hours=1)
 
 
-class TestIsAware():
+class TestIsAware:
     def test_naive(self):
         assert utils.is_aware(dt.datetime.now()) is False
 
